﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Services;
using System.Data.Services.Common;
using System.Data.Services.Client;
using System.ServiceModel.Web;
using DoodleJoyModels;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.ComponentModel;
using System.Reflection;

namespace DoodleJoyCloud_Web
{
    public class DoodleData : IUpdatable
    {
        private TableServiceContext context;
        public TableServiceContext GetContext()
        {
            return CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString")).CreateCloudTableClient().GetDataServiceContext();
        }

        public DoodleData()
        {
            context = GetContext();
        }

        public IQueryable<Subscription> Subscriptions
        {
            get { return GetContext().CreateQuery<Subscription>("Subscription").AsTableServiceQuery(); }
        }

        public IQueryable<Doodle> Doodles
        {
            get
            {
                return GetContext().CreateQuery<Doodle>("Doodles").Take(10).AsTableServiceQuery();
            }
        }

        public object CreateResource(string containerName, string fullTypeName)
        {
            if (fullTypeName == typeof(Doodle).FullName)
            {
                var doodle = new Doodle();
                context.AddObject("Doodles", doodle);
                return doodle;
            }
            else if (fullTypeName == typeof(Subscription).FullName)
            {
                var subscription = new Subscription();
                context.AddObject("Subscriptions", subscription);
                return subscription;
            }
            else
            {
                throw new ArgumentException(string.Format("Unexpected type '{0}'", fullTypeName));
            }
        }

        public object ResolveResource(object resource)
        {
            return resource;
        }

        public void SaveChanges()
        {
            try
            {
                context.SaveChangesWithRetries();
            }
            catch (DataServiceRequestException) // should check that the error is a conflict
            {
                foreach (var subscription in context.Entities.Select(e => e.Entity).OfType<Subscription>())
                {
                    context.Detach(subscription);
                    context.AttachTo("Subscriptions", subscription, "*");
                    context.UpdateObject(subscription);
                }
                context.SaveChangesWithRetries();
            }

            var queue = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString")).CreateCloudQueueClient().GetQueueReference("newdoodles");
            foreach (var doodle in context.Entities.Select(e => e.Entity).OfType<Doodle>())
            {
                queue.AddMessage(new CloudQueueMessage(doodle.RowKey));
            }
        }

        public void SetValue(object targetResource, string propertyName, object propertyValue)
        {
            if (targetResource is Subscription || (propertyName != "PartitionKey" && propertyName != "RowKey"))
            {
                if (propertyName == "ImageUri")
                {
                    // change the URL to use the CDN, commented out
                    //propertyValue = new UriBuilder((string)propertyValue) { Scheme = "http", Port = 80, Host = "cdn.doodlejoy.com", Query = null }.Uri.AbsoluteUri;
                    propertyValue = new UriBuilder((string)propertyValue) { Query = null }.Uri.AbsoluteUri;
                }
                targetResource.GetType().GetProperty(propertyName).SetValue(targetResource, propertyValue, null);
            }
        }

        #region notimplemented
        public void AddReferenceToCollection(object targetResource, string propertyName, object resourceToBeAdded)
        {
            throw new NotImplementedException();
        }

        public void ClearChanges()
        {
            throw new NotImplementedException();
        }

        public void DeleteResource(object targetResource)
        {
            throw new NotImplementedException();
        }

        public object GetResource(IQueryable query, string fullTypeName)
        {
            throw new NotImplementedException();
        }

        public object GetValue(object targetResource, string propertyName)
        {
            throw new NotImplementedException();
        }

        public void RemoveReferenceFromCollection(object targetResource, string propertyName, object resourceToBeRemoved)
        {
            throw new NotImplementedException();
        }

        public object ResetResource(object resource)
        {
            throw new NotImplementedException();
        }

        public void SetReference(object targetResource, string propertyName, object propertyValue)
        {
            throw new NotImplementedException();
        }
        #endregion
    }

    public class DoodleService : DataService<DoodleData>
    {
        [WebGet]
        public string GetBlobUri()
        {
            var blob =
                CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"))
                .CreateCloudBlobClient().GetContainerReference("images")
                .GetBlobReference(Guid.NewGuid().ToString());
            var sas = blob.GetSharedAccessSignature(new SharedAccessPolicy
            {
                Permissions = SharedAccessPermissions.Write,
                SharedAccessExpiryTime = DateTime.UtcNow + TimeSpan.FromMinutes(5)
            });

            return new UriBuilder(blob.Uri) { Query = sas.TrimStart('?') }.Uri.AbsoluteUri;
        }

        public static void InitializeService(DataServiceConfiguration config)
        {
            config.SetEntitySetAccessRule("Doodles", EntitySetRights.All);
            config.SetEntitySetAccessRule("Subscriptions", EntitySetRights.AllWrite);
            config.SetServiceOperationAccessRule("GetBlobUri", ServiceOperationRights.All);
            config.UseVerboseErrors = true;
        }
    }
}