﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Notification;
using DoodleJoyModels;
using Microsoft.Phone.Reactive;
using System.Data.Services.Client;
using System.IO.IsolatedStorage;

namespace DoodleJoyPhone
{
    public partial class App : Application
    {
        private static MainViewModel viewModel = null;

        public static MainViewModel ViewModel
        {
            get
            {
                if (viewModel == null)
                    viewModel = new MainViewModel();

                return viewModel;
            }
        }

        public PhoneApplicationFrame RootFrame { get; private set; }

        private HttpNotificationChannel channel;

        private void Subscribe(Uri uri)
        {
            if (!IsolatedStorageSettings.ApplicationSettings.Contains("guid"))
            {
                IsolatedStorageSettings.ApplicationSettings["guid"] = Guid.NewGuid().ToString();
            }
            var svc = new DoodleData(ViewModel.ServiceUri);
            svc.AddToSubscriptions(new Subscription
            {
                PartitionKey = (string)IsolatedStorageSettings.ApplicationSettings["guid"],
                RowKey = string.Empty,
                NotificationUri = uri.AbsoluteUri
            });
            Observable.FromAsyncPattern<DataServiceResponse>(svc.BeginSaveChanges, svc.EndSaveChanges)().ObserveOnDispatcher().Subscribe(_ =>
            {
            },
            ex =>
            {
                MessageBox.Show(ex.ToString());
            });
        }

        private void SetUpDelegates()
        {
            channel.ChannelUriUpdated += (s, e) =>
                {
                    Subscribe(e.ChannelUri);
                };
            channel.ShellToastNotificationReceived += (s, e) =>
                {
                    ViewModel.LoadData();
                };
        }

        public App()
        {
            UnhandledException += Application_UnhandledException;

            channel = HttpNotificationChannel.Find("notifications");
            if (channel == null)
            {
                channel = new HttpNotificationChannel("notifications");
                SetUpDelegates();
                channel.Open();
            }
            else
            {
                SetUpDelegates();
                if (channel.ChannelUri != null)
                {
                    Subscribe(channel.ChannelUri);
                }
            }
            if (!channel.IsShellTileBound)
            {
                // This points to dev storage.  Consider using <account>.blob.core.windows.net or a custom domain name or CDN URL.
                channel.BindToShellTile(new System.Collections.ObjectModel.Collection<Uri>(new[] { new Uri("http://127.0.0.1:10000") }));
            }
            if (!channel.IsShellToastBound)
            {
                channel.BindToShellToast();
            }

            InitializeComponent();

            InitializePhoneApplication();
        }

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
            // Ensure that application state is restored appropriately
            if (!App.ViewModel.IsDataLoaded)
            {
                App.ViewModel.LoadData();
            }
        }

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
            // Ensure that required application state is persisted here.
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

        // Code to execute if a navigation fails
        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // A navigation has failed; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        #region Phone application initialization

        // Avoid double-initialization
        private bool phoneApplicationInitialized = false;

        // Do not add any additional code to this method
        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
                return;

            // Create the frame but don't set it as RootVisual yet; this allows the splash
            // screen to remain active until the application is ready to render.
            RootFrame = new PhoneApplicationFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;

            // Handle navigation failures
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;

            // Ensure we don't initialize again
            phoneApplicationInitialized = true;
        }

        // Do not add any additional code to this method
        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            // Set the root visual to allow the application to render
            if (RootVisual != RootFrame)
                RootVisual = RootFrame;

            // Remove this handler since it is no longer needed
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion
    }
}