﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Windows.Ink;
using System.Windows.Media.Imaging;
using DoodleJoyModels;
using Microsoft.Phone.Reactive;
using System.Data.Services.Client;
using System.IO;
using System.Windows.Navigation;

namespace DoodleJoyPhone
{
    public partial class CreatePage : PhoneApplicationPage
    {
        public CreatePage()
        {
            this.Loaded += new RoutedEventHandler(CreatePage_Loaded);
            InitializeComponent();
        }

        static HttpWebRequest CreateWebRequest(string uri, string method, string contentType)
        {
            var req = (HttpWebRequest)WebRequest.Create(uri);
            req.Method = method;
            req.ContentType = contentType;
            return req;
        }

        static Stroke CreateStroke(TouchPoint pt)
        {
            var stroke = new Stroke();
            stroke.DrawingAttributes.Color = Colors.White;
            stroke.DrawingAttributes.Width = 20;
            stroke.DrawingAttributes.Height = 20;
            stroke.StylusPoints.Add(new StylusPoint(pt.Position.X, pt.Position.Y));
            return stroke;
        }

        void CreatePage_Loaded(object sender, RoutedEventArgs e)
        {
            var sendClicks = from click in Observable.FromEvent<EventArgs>(ApplicationBar.Buttons[0], "Click")
                             let svc = new DoodleData(App.ViewModel.ServiceUri)
                             let bitmap = new WriteableBitmap(Surface, new TranslateTransform())
                             let title = TitleBox.Text
                             from uri in Observable.FromAsyncPattern<Uri, IEnumerable<string>>(svc.BeginExecute<string>, svc.EndExecute<string>)(new Uri("GetBlobUri", UriKind.Relative)).Select(s => s.Single())
                             let req = CreateWebRequest(uri, "PUT", "image/jpeg")
                             from _ in Observable.FromAsyncPattern<Stream>(req.BeginGetRequestStream, req.EndGetRequestStream)().Do(stream =>
                             {
                                 bitmap.SaveJpeg(stream, bitmap.PixelWidth, bitmap.PixelHeight, 0, 100);
                                 stream.Close();
                             })
                             from response in Observable.FromAsyncPattern<WebResponse>(req.BeginGetResponse, req.EndGetResponse)().Do(__ =>
                                 svc.AddToDoodles(new Doodle
                                 {
                                     PartitionKey = string.Empty,
                                     RowKey = string.Empty,
                                     Title = title,
                                     ImageUri = uri
                                 }))
                             select Observable.FromAsyncPattern<DataServiceResponse>(svc.BeginSaveChanges, svc.EndSaveChanges)();

            sendClicks.ObserveOnDispatcher().Subscribe(_ => NavigationService.GoBack(), ex => MessageBox.Show(ex.ToString()));

            var surfaceTouchPoints = from args in Observable.FromEvent<TouchFrameEventHandler, TouchFrameEventArgs>(h => new TouchFrameEventHandler(h), h => Touch.FrameReported += h, h => Touch.FrameReported -= h)
                                     let pt = args.EventArgs.GetPrimaryTouchPoint(Surface)
                                     where pt.Position.X >= 0 && pt.Position.Y >= 0 && pt.Position.X < Surface.ActualWidth && pt.Position.Y < Surface.ActualHeight
                                     select pt;

            var upTouchPoints = from pt in surfaceTouchPoints where pt.Action == TouchAction.Up select pt;
            var moveTouchPoints = from pt in surfaceTouchPoints where pt.Action == TouchAction.Move select pt;
            var downTouchPoints = from pt in surfaceTouchPoints where pt.Action == TouchAction.Down select pt;

            (from pt in downTouchPoints select CreateStroke(pt))
                .TakeUntil(Observable.FromEvent<NavigatingCancelEventArgs>(NavigationService, "Navigating")).ObserveOnDispatcher().Subscribe(stroke =>
                {
                    Surface.Strokes.Add(stroke);
                    moveTouchPoints.ObserveOnDispatcher().TakeUntil(upTouchPoints).Subscribe(pt =>
                        stroke.StylusPoints.Add(new StylusPoint(pt.Position.X, pt.Position.Y))
                    );
                });

            Observable.FromEvent<RoutedEventArgs>(TitleBox, "GotFocus").Where(_ => TitleBox.Text == "my doodle").Subscribe(_ => TitleBox.SelectAll());
        }
    }
}